<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:fo="http://www.w3.org/1999/XSL/Format"
                xmlns:saxon="http://icl.com/saxon"
                exclude-result-prefixes="xsl fo saxon">

  <xsl:output
      method="xml"
      indent="yes"
      omit-xml-declaration="yes"/>

  <xsl:template match="/">
    <html>
      <head>
        <title>OpenLink AJAX Toolkit</title>
        <link rel="STYLESHEET" type="text/css" href="oat.css" />
      </head>
      <body>
        <div id="page">
          <div id="oatnav">
            <a href="index.html">Index</a> | 
            <a href="http://oat.openlinksw.com/" target="_new">OAT homepage</a> | 
            <a href="http://www.openlinksw.com/" target="_new">OpenLink Software</a>
          </div>

          <xsl:choose>                                                    
            <!--pull out first heading before the ToC -->
            <xsl:when test="count(//h1[1])&gt;0">
              <xsl:apply-templates select="//h1[1]" mode="head" />
            </xsl:when>
            <xsl:otherwise>
              <xsl:apply-templates select="//h2[1]" mode="head" />
            </xsl:otherwise>
          </xsl:choose>

          <xsl:apply-templates select="//div[@class='topic-text']" />

          <div id="oatcopy">
            Copyright (C) 2005-2009
            <a href="http://www.openlinksw.com/">OpenLink Software</a>
          </div>

        </div>
      </body>
    </html>
  </xsl:template>

  <!-- tidy up empty paragraphs -->
  <xsl:template match="p[normalize-space(.)='']" />

  <!-- Remove display of Category paragraphs -->
  <xsl:template match="p[starts-with(normalize-space(.),'Category')]" />
  <xsl:template match="a[starts-with(normalize-space(.),'Category')]" />
  <xsl:template match="text()[starts-with(normalize-space(.),'Category')]" />

  <xsl:template match="a[normalize-space(./text())='?']" />

  <!-- replace whole head element -->
  <xsl:template match="//head" />

  <!-- copy the ToC -->
  <xsl:template match="//div[@class='wikitoc']">
    <div class="oatwikitoc">
      <div class="wikitoc">
        <xsl:apply-templates />
      </div>
    </div>
  </xsl:template>

  <xsl:template match="//body">
    <div id="oatnav">
      <a href="index.html">Index</a> | 
      <a href="http://oat.openlinksw.com/" target="_new">OAT homepage</a> | 
      <a href="http://www.openlinksw.com/" target="_new">OpenLink Software</a>
    </div>

    <!-- handle majority of page content -->
    <xsl:apply-templates />
  </xsl:template>

  <!-- remove first heading from page-body now it's above the ToC -->
  <xsl:template match="//h1[1]" />

  <xsl:template match="h1|h2" mode="head">
    <h1><xsl:value-of select="." /></h1>
  </xsl:template>

  <!-- handle links -->

  <!-- convert inline links to filenames -->
  <xsl:template match="a[starts-with(@href, 'http://wiki.usnet.private') 
                       and contains(@href,'/OAT')]">
    <a>
      <xsl:attribute name="href">OAT<xsl:value-of select="substring-after(@href, 'OATWikiWeb/OAT')" />.html</xsl:attribute>
      <xsl:apply-templates />
    </a>
  </xsl:template>

  <!-- Unlink other references to usnet.private -->
  <xsl:template match="a[starts-with(@href, 'http://wiki.usnet.private') and not(contains(@href, '/OATDOC'))]">
    <xsl:apply-templates />
  </xsl:template>

  <!-- Replace links to index with index.html -->
  <xsl:template match="a[contains(@href, 'OATDOCIndex')]">
    <a href="index.html">Index</a>
  </xsl:template>

  <!-- Remove links to missing pages -->
  <xsl:template match="a[normalize-space(./text())='?']">
  </xsl:template>

  <!-- Remove links to nowhere -->
  <xsl:template match="a[normalize-space(@href)='#']">
  </xsl:template>
  <xsl:template match="a[normalize-space(@href)='?']">
  </xsl:template>

  <!-- Ensure empty a tags don't get truncated to self-closing -->
  <xsl:template match="a[normalize-space(./text())='']">
    <xsl:element name="a">
      <xsl:apply-templates select="@*" mode="attrs" />
      <xsl:text> 
      </xsl:text>
    </xsl:element>
  </xsl:template>

  <!-- Remove meta-headers -->
  <xsl:template match="meta[@name='generator']" />

  <!-- replace nav and copy bars -->
  <xsl:template match="div[@id='oatnav']" />

  <!-- Replace the copyright block -->
  <xsl:template match="div[@id='oatcopy']" />

  <!-- Remove spurious br tags within headings -->
  <xsl:template match="h1/br" />
  <xsl:template match="h2/br" />
  <xsl:template match="h3/br" />
  <xsl:template match="h4/br" />
  <xsl:template match="h5/br" />
  <xsl:template match="h6/br" />

  <!-- Handle spurious to-be-done paragraphs -->
  <xsl:template match="p[contains(.//text(), 'ToBeDone')]" priority="0">
    <div class="tdl">
      <xsl:apply-templates mode="tbd" />
    </div>
  </xsl:template>

  <!-- Handle ToBeDone paragraphs -->
  <xsl:template match="*" mode="tbd">
    <xsl:element name="{name(.)}">
      <xsl:apply-templates select="@*" mode="attrs" />
      <xsl:apply-templates />
    </xsl:element>
  </xsl:template>

  <!-- remove links from ToBeDone paragraphs -->
  <xsl:template match="a" mode="tbd" />

  <!-- Generic: just copy most elements -->
  <xsl:template match="*">
    <xsl:element name="{name(.)}">
      <xsl:apply-templates select="@*" mode="attrs" />
      <xsl:apply-templates />
    </xsl:element>
  </xsl:template>

  <xsl:template match="@*" mode="attrs" priority="0">
    <xsl:attribute name="{name(.)}"><xsl:value-of select="." /></xsl:attribute>
  </xsl:template>

</xsl:stylesheet>
