#!/bin/zsh

BASE="http://wiki.usnet.private/dataspace/dav/wiki/OATWikiWeb/"
START="OATDOCIndex"
PLAIN="?command=plain-html"
SOURCE="?command=text"
DOCBOOK="?command=docbook"
EDIT="?command=edit"
FILTER="OAT"

notify() {
    echo "$*"
}

grabpageplain() {
    pg="$1"
    if [ ! -f "$pg.html" ]
    then
        notify "Grabbing $pg"
        curl -s "$BASE$pg$PLAIN" > foo.html
        curl -s "$BASE$pg$SOURCE" > "$pg.txt"
        curl -s "$BASE$pg$DOCBOOK" > "$pg.docbook"
        cp foo.html backup/"$pg.html"
        perl -npi.bak -e 's#<html xmlns="http://www.w3.org/1999/xhtml">#<html>#; s#<br>#<br />#goi' foo.html
        tidy -i -c -m -asxhtml foo.html |& grep -Ei 'errors'
        perl -npi.bak -e 's#<html xmlns="http://www.w3.org/1999/xhtml">#<html>#; s#<br>#<br />#goi' foo.html
        mv foo.html "$pg.html" || notify "$pg failed to produce foo.html"
    fi
}

pagelinks() {
    pg="$1:r"
    notify "Finding links within $pg.links"
    xmlstarlet sel -t -m '//a' -v '@href' -n "$pg.html" | sed 's#^'"$BASE"'##' | sed -e 's/[\?\#].*$//' | sed 's/OATWikiWeb//' | grep -i "^$FILTER" > "$pg.links"
}

recurse() {
    foreach i (*.links) {
        echo "Examining links in $i"
        while read p
        do
            [ ! -f "$p.html" ] && grabpageplain $p && pagelinks $p
        done < $i
    }
}

inpagefixlinks() {
    pg="$1:r"
    pgh="$pg.html"
    notify "Running XSLT on $pg" 
    sabcmd fix-links.xsl "$pgh" foo.html && mv foo.html "$pgh"
}

################
#
# main bit here
#

# clean everything
rm -f *.html 
rm -f *.links 
rm -f *.txt 
rm -f *.bak
rm -f *.docbook

#prepare backup dir for debugging against
mkdir -p backup 2> /dev/null

#Grab pages to start with
notify "Starting with [$START]"
grabpageplain "$START"
pagelinks "$START"

#recursive down to 3 levels
repeat 3 recurse

#fix links and do other tweaks in XSLT within pages
foreach i (*.html) {
    inpagefixlinks $i
}

perl -npi.bak -e 's#OATDOCIndex.html#index.html#goi; s#OATDOCIndex#index#goi' *.html

cp OATDOCIndex.html index.html && rm OATDOCIndex.html
