/* music ontology */

var TL = {
	obj:[],

	/* create timeline; add bands based on triple content */
	init:function() {
		var options1 = {
			formatter:false,
			resize:false,
			timeStepOverride:"_fiveseconds",
			timeTitleOverride:function(date) {
				return Data.formatdate(date);
			},
			timeLabelOverride:function(date) {
				return Data.formatdate(date);
			}
		}
		var options2 = {
			formatter:false,
			resize:false,
			timeStepOverride:"_seconds",
			timeTitleOverride:function(date) {
				return Data.formatdate(date);
			},
			timeLabelOverride:function(date) {
				return Data.formatdate(date);
			}
		}

		TL.obj = [];
		OAT.Dom.clear("tl1");
		var tl1 = new OAT.Timeline("tl1",options1);
		TL.obj.push(tl1);
		OAT.Event.attach(window,"resize",tl1.sync); 
		OAT.Dom.clear("tl2");
		var tl2 = new OAT.Timeline("tl2",options2);
		TL.obj.push(tl2);
		OAT.Event.attach(window,"resize",tl2.sync);
	},

	/* plot events */
	add:function(tl,triples) { 
		var colors = ["#cf6","#fb9","#887fff","#ff997f","#66ffe6","#96f","#7fff66"];
		var cindex = 1;

		/* root band */
		var root = Data.findP(triples,"time");

		if(!root) {
			var note = new OAT.Notify();
			note.send("File does not contain any data that could be displayed. Please check your URL.");
			return;
		}

		tl.addBand(root[0],colors[0]);

		/* find duration otherwise set it to 0sec */
		var dur = Data.findSP(triples,root[2],"durationXSD");
		dur = (dur)? dur[2] : "P0S";

		var start = Data.date(false);
		var end = Data.date(dur);

		var div = OAT.Dom.create("div");
		if (dur != "P0S") { div.innerHTML = "Duration"; }
		tl.addEvent(root[0],start,end,div,"#888");

		var types = ["HarmonicChange","KeyEvent","ChordEvent","StructuralSegment","Onset","FeatureEvent"];
		for (var i=0;i<types.length;i++) {
			var type = types[i];
			var events = Data.grepPO(triples,"type",type);
			if(events.length) {
				tl.addBand(type,colors[cindex++]);
				for (var j=0;j<events.length;j++) { TL.addEvent(tl,triples,events[j][0],type);	}
			}
		}
		
		tl.draw();
	},
	
	/* add a single event to timeline */
	addEvent:function(tl,triples,e,bandtype) {

		/* first, label the interval */
		var label;

		/* try event label name */
		label = Data.findSP(triples,e,"label");

		/* try key name */
		if (!label) { label = Data.findSP(triples,e,"key"); }

		/* try timelabel name */
		if (!label) { label = Data.findSP(triples,time,"label"); }

		label = (label)? label[2] : " ";

		var div = OAT.Dom.create("div");
		div.innerHTML = label;

		var time = Data.findSP(triples,e,"time")[2];
		var type = Data.findSP(triples,time,"type")[2];

		if (type == "Instant") { 
			/* instant temporal change */
			var instant = Data.findSP(triples,time,"atDuration");
			instant = (instant)? instant[2] : "P0S";
			var start = Data.date(instant);
			var end = new Date(start.getTime + Data.duration("P0.001S"));
		} else {
			/* time interval */

			/* start time first */
			var start_instant = Data.findSP(triples,time,"beginsAtDuration");
			start_instant = (start_instant)? start_instant[2] : "P0S";
			var start = Data.date(start_instant);
			var end = Data.date(start_instant);

			/* end time - explicit instant */
			var end_instant = Data.findSP(triples,time,"endsAtDuration");
			end_instant = (end_instant)? end_instant[2] : false;

			/* end time - interval duration */
			var duration = Data.findSP(triples,time,"durationXSD");
			duration = (duration)? duration[2] : false;

			if (end_instant) {
				end	= Data.date(end_instant);
			} else if (duration) {	
				end = new Date(start.getTime() + Data.duration(duration));
			}
		}
		
		tl.addEvent(bandtype,start,end,div,"#888");
	}
}

var Data = {
	/* simplify uri's */
	simplify:function(triple) {
		var s = triple[0];
		var p = triple[1];
		var o = triple[2];

		/* we trim namespace URIs */
		var trim = function(str) {
			if(str.match(/^http/i)) {
				str = str.match(/([^\/#]*)$/)[1];
			}
			return str;
		}

		s = trim(s); p = trim(p); o = trim(o);
		return [s,p,o];
	},

	/* get triples from xml and simplify them */
	parse:function(xmlDoc) {
		var triples = OAT.RDF.toTriples(xmlDoc);
		var t = [];
		for (i=0;i<triples.length;i++) {
			t.push(Data.simplify(triples[i]));
		}
		return t;
	},

	/* return total duration time in miliseconds */
	duration:function(str) {
		/* xsd duration format: PnYnMnDTnHnMnS */
		var total = 0;
		var time = str.match(/(?:([\d\.]+)M)?(?:([\d\.]+)S)?$/);
		if (time[1]) {
			total += parseFloat(time[1]) * 60 * 1000;
		}
		if (time[2]) {
			total += parseFloat(time[2]) * 1000;
		}
		return total;
	},

	/* return event time in unified format (miliseconds) */
	date:function(str) {
		var d = new Date();
		d.setHours(0)
		d.setMinutes(0);
		d.setSeconds(0);
		d.setMilliseconds(0);
		if (!str) { return d; }
		d.setMilliseconds(Data.duration(str));
		return d;
	},

	formatdate:function(date) {
		str  = "";
		str += date.getMinutes().toString().leadingZero(2);
		str += ":";
		str += date.getSeconds().toString().leadingZero(2);
		str += ".";
		str += date.getMilliseconds().toString().leadingZero(3);
		return str;
	},

	/* triples search */
	grepS:function(triples,s) {
		var result = [];
		for (var i=0;i<triples.length;i++) {
			var t = triples[i];
			if (t[0] == s) { result.push(t); }
		}
		return result;
	},
	grepP:function(triples,p) {
		var result = [];
		for (var i=0;i<triples.length;i++) {
			var t = triples[i];
			if (t[1] == p) { result.push(t); }
		}
		return result;
	},
	grepPO:function(triples,p,o) {
		var result = [];
		for (var i=0;i<triples.length;i++) {
			var t = triples[i];
			if (t[1] == p && t[2] == o) { result.push(t); }
		}
		return result;
	},
	grepSP:function(triples,s,p) {
		var result = [];
		for (var i=0;i<triples.length;i++) {
			var t = triples[i];
			if (t[1] == p && t[0] == s) { result.push(t); }
		}
		return result;
	},

	findS:function(triples,s) {
		var result = false;
		for (var i=0;i<triples.length;i++) {
			var t = triples[i];
			if (t[0] == s) { result = t; break; }
		}
		return result;
	},

	findP:function(triples,p) {
		result = false;
		for (var i=0;i<triples.length;i++) {
			var t = triples[i];
			if (t[1] == p) { result = t; break; }
		}
		return result;
	},

	findSP:function(triples,s,p) {
		result = false;
		for (var i=0;i<triples.length;i++) {
			var t = triples[i];
			if (t[1] == p && t[0] == s) { result = t; break; }
		}
		return result;
	}
}

function load() {
	/* 
		1. fetch data
		2. parse it
		3. init timeline
		4. plot events 
	*/
	
	var callback = function(xmlDoc) {
		/* #2 */
		var triples = Data.parse(xmlDoc);
		/* #3 */
		TL.init();
		/* #4 */
		TL.add(TL.obj[0],triples);
		TL.add(TL.obj[1],triples);
	}
	
	/* #1 */
	var url = "ajax.php" + "?q=" + $v("url");
	OAT.AJAX.GET(url,false,callback,{type:OAT.AJAX.TYPE_XML});
}

function init() {
	OAT.Event.attach("btn","click",load);
	OAT.Event.attach("url","keypress",function(e) {
		if (e.keyCode == 13) { load(); }
	});
	OAT.Dom.hide("throbber");
	OAT.AJAX.startRef = function() { OAT.Dom.show("throbber"); }
	OAT.AJAX.endRef = function() { OAT.Dom.hide("throbber"); }
}
