#
#  Service.asmx
#
#  $Id: Service.asmx,v 1.3 2012/06/29 00:23:09 source Exp $
#
#  XMLA provider for ODBC
#
#  Copyright (C) 2008-2012 OpenLink Software.
#  All Rights Reserved.
#
#  The copyright above and this notice must be preserved in all
#  copies of this source code.	The copyright above does not
#  evidence any actual or intended publication of this source code.
#
#  This is unpublished proprietary trade secret of OpenLink Software.
#  This source code may not be copied, disclosed, distributed, demonstrated
#  or licensed except as authorized by OpenLink Software.
#

include common.tcl
include forms.tcl
include wizard.tcl
include auth.tcl
include txact.tcl
include odbcdsn.tcl
include navigator.tcl

# undefined
set undef "UNDEF"
set _catalogHack 0
set _close_soap_answer 1



proc pr_SoapHeader {} {
 html {<?xml version="1.0" encoding="utf-8"?>}
}


proc pr_SoapEnvelopeBegin {} {
 html {<soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema">}
}


proc pr_SoapEnvelopeEnd {} {
 html </soap:Envelope>      
}


proc pr_SoapBodyEmpty {cmd} {
 html {<soap:Body><}
 html ${cmd}Response 
 html { xmlns="urn:schemas-microsoft-com:xml-analysis"><return><root xmlns="urn:schemas-microsoft-com:xml-analysis:empty" />}
 html {</return></}
 html ${cmd}Response
 html {></soap:Body>}
}


proc pr_SoapBodyBegin {cmd} {
 html {<soap:Body><}
 html ${cmd}Response 
 html { xmlns="urn:schemas-microsoft-com:xml-analysis"><return><root xmlns="urn:schemas-microsoft-com:xml-analysis:rowset"><s:schema xmlns:sql="urn:schemas-microsoft-com:xml-sql" xmlns:s="http://www.w3.org/2001/XMLSchema">}
}


proc pr_SoapBodyEnd {cmd} {
 html {</root></return></}
 html ${cmd}Response
 html {></soap:Body>}
}



proc send_Empty { cmd } {
  global _close_soap_answer
  set _close_soap_answer 0
  html_clear
  pr_SoapHeader
  pr_SoapEnvelopeBegin
  pr_SoapBodyEmpty $cmd
  pr_SoapEnvelopeEnd
}


proc send_Error { errMess } {
  global _close_soap_answer

#HTTP/1.1 500 Internal Server Error..      

  set _close_soap_answer 0

  html_clear

  pr_SoapHeader
  pr_SoapEnvelopeBegin

  html <soap:Body><soap:Fault><faultcode>soap:Server</faultcode><faultstring>
  html $errMess
  html </faultstring><detail /></soap:Fault></soap:Body>

  pr_SoapEnvelopeEnd
}


proc pr_ColDef { cname ctype} {
  html <s:element sql:field="$cname" name="$cname" type="s:$ctype" />
}


proc pr_ColsDef { COLname COLtype } {
    set c [llength $COLname]

    html <s:complexType name="row"><s:choice minOccurs="0" maxOccurs="unbounded">
    for {set i 1} {$i < $c} {incr i} {
	pr_ColDef [lindex $COLname $i]  [odbc2xml [lindex $COLtype $i]]
    }
    html </s:choice></s:complexType></s:schema>
}



proc pr_ColData { cname cdata} {
   html <$cname>
   html -escape $cdata
   html </$cname>
}


proc pr_ColData2 { cname cdata} {
 if {1==[lindex $cdata 0]} {
   html <$cname>
   html -escape [lindex $cdata 1]
   html </$cname>
 } else {
   html <$cname />
 }
}


proc odbc2xml { type } {
  switch -- $type {
#       BIGINT = (-5),
    -5 { return "int" }
#       BINARY = (-2),
    -2 { return "base64Binary" }
#       BIT = (-7),
    -7 { return "boolean" }
#       CHAR = 1,
    1  { return "string" }
#       DATE = 9,
    9  { return "any" }
#       DECIMAL = 3,
    3  { return "decimal" }
#       DOUBLE = 8,
    8  { return "double" }
#       GUID = (-11),
    -11 { return "any" }
#       INTEGER = 4,
    4  { return "int" }
#       LONGVARBINARY = (-4),
    -4 { return "base64Binary" }
#       LONGVARCHAR = (-1),
    -1 { return "string" }
#       NUMERIC = 2,
    2  { return "decimal" }
#       FLOAT = 6,
    6  { return "float" }
#       REAL = 7,
    7  { return "float" }
#       SMALLINT = 5,
    5  { return "short" }
#       TIME = 10,
    10 { return "any" }
#       TIMESTAMP = 11,
    11 { return "any" }
#       TINYINT = (-6),
    -6 { return "short" }
#       TYPE_DATE = 91,
    91 { return "any" }
#       TYPE_TIME = 92,
    92 { return "any" }
#       TYPE_TIMESTAMP = 93,
    93 { return "any" }
#       VARBINARY = (-3),
    -3 { return "base64Binary" }
#       VARCHAR = 12,
    12 { return "string" }
#       WCHAR = (-8),
    -8 { return "string" }
#       WLONGVARCHAR = (-10),
    -10 { return "string" }
#       WVARCHAR = (-9),
    -9 { return "string" }

    default { return "any" }
  }
}


proc getOpt { lst name def } {
    set pos [lsearch $lst $name]
    if {$pos > 0} { 
      return [lindex $lst [incr pos]] 
    } else { 
      return $def
    }
}



proc parse_cmd {req param props restr} {
    upvar 1 $req myReq  $param myParam $props myProps $restr myRestr
    global undef
    set reqBody 0

    set cl_req [xmldoc parsememory cl_req [req args]]

    for {set xmlRoot [cl_req getRootElement xmlRoot] } { ![xmlRoot isNULL] } { xmlRoot next } {
      if {[xmlRoot name] == "Envelope"} { 
           set reqBody [xmlRoot children reqBody]
           while { [reqBody isBlank] } {
             set reqBody [reqBody next]
           }
           break 
      }
    }

    if { ![reqBody isNULL] && [reqBody name]=="Body" } {
      set node [reqBody children node]
      if { ![node isNULL]} {

        while { [node isBlank] } {
          set node [node next]
        }

        set myReq [node name]

        if {[node name]=="Discover"} {
          # Discover request
          set nchild [node children nchild]
          while { ![nchild isNULL]} {
            if {[nchild name]=="RequestType"} {
              set myParam [nchild characters]

            } elseif {[nchild name]=="Properties"} {
              nchild children nproplist
              while { [nproplist isBlank] } {
                set nproplist [nproplist next]
              }
              if { ![nproplist isNULL] && [nproplist name]=="PropertyList" } {
                set nprops [nproplist children nprops]
                while { ![nprops isNULL]} {
                  lappend myProps [nprops name] [nprops characters]
                  nprops next
                }
              }
            } elseif {[nchild name]=="Restrictions"} {
              nchild children nproplist
              while { [nproplist isBlank] } {
                set nproplist [nproplist next]
              }
              if { ![nproplist isNULL] && [nproplist name]=="RestrictionList" } {
                set nprops [nproplist children nprops]
                while { ![nprops isNULL]} {
                  lappend myRestr [nprops name] [nprops characters]
                  nprops next
                }
              }
            }

            nchild next
          }

        } elseif {[node name]=="Execute"} {

          # Execute request
          set nchild [node children nchild]
          while { [nchild isBlank] } {
            set nchild [nchild next]
          }

          while { ![nchild isNULL]} {

            if {[nchild name]=="Command"} {
              set nchild0 [nchild children nchild0]
              while { [nchild0 isBlank] } {
                set nchild0 [nchild0 next]
              }

              while { ![nchild0 isNULL]} {
                if {[nchild0 name]=="Statement"} {
                  set myParam [nchild0 characters]
                }
                nchild0 next
              }

            } elseif {[nchild name]=="Properties"} {
              nchild children nproplist
              while { [nproplist isBlank] } {
                set nproplist [nproplist next]
              }
              if { ![nproplist isNULL]} {
                set nprops [nproplist children nprops]
                while { ![nprops isNULL] && [nproplist name]=="PropertyList" } {
                  lappend myProps [nprops name] [nprops characters]
                  nprops next
                }
              }
            } elseif {[nchild name]=="Restrictions"} {
              nchild children nproplist
              while { [nproplist isBlank] } {
                set nproplist [nproplist next]
              }
              if { ![nproplist isNULL]} {
                set nprops [nproplist children nprops]
                while { ![nprops isNULL] && [nproplist name]=="RestrictionList" } {
                  lappend myRestr [nprops name] [nprops characters]
                  nprops next
                }
              }
            }

            nchild next
          }

        } else {
          return 0
        }
      }
    } else {
      return 0
    }
  return 1
}


proc load_ColsDef {st COLname COLtype} {
    upvar 1 $COLname _COLname  $COLtype _COLtype

    set c [st numcols]
    if {0 != $c} {
      set _COLname "."
      set _COLtype "."
      for {set i 1} {$i <= $c} {incr i} {
	set cname [st colattr $i name]
	set ctype [st colattr $i type]
	lappend _COLname $cname
	lappend _COLtype $ctype
      }
    }
}


proc pr_DSN {dname descr info pname ptype auth} {
 html <row>
 pr_ColData "DataSourceName" $dname
 pr_ColData "DataSourceDescription" $descr
 pr_ColData "DataSourceInfo" $info
 pr_ColData "ProviderName" $pname
 pr_ColData "ProviderType" $ptype
 pr_ColData "AuthenticationMode" $auth
 html </row>
}





proc send_Datasources { _cmd } {

  html <s:complexType name="row"><s:choice minOccurs="0" maxOccurs="unbounded">
  pr_ColDef "DataSourceName" "string"
  pr_ColDef "DataSourceDescription" "string"
  pr_ColDef "DataSourceInfo" "string"
  pr_ColDef "ProviderName" "string"
  pr_ColDef "ProviderType" "string"
  pr_ColDef "AuthenticationMode" "string"
  html </s:choice></s:complexType></s:schema>

  set dsnList [ GetDSNList ]
  set c [llength $dsnList] 
  for {set i 0} {$i < $c} {incr i} {
    pr_DSN dsn$i "ODBC datasource" [lindex $dsnList $i] "iODBCXmlaProvider" "TDP" "Authenticated"
  }
}



proc connect_db { db props } {
  global _catalogHack

    set DSN [getOpt $props "DataSourceInfo" ""]
    set UID [getOpt $props "UserName" ""]
    set PWD [getOpt $props "Password" ""]
    set CAT [getOpt $props "Catalog" ""]

    database connect db $DSN $UID $PWD

    set _dbms [db getinfo dbms_name]
    if { $_dbms == "ACCESS" } {
      set $_catalog_hack "1"
    }
}




proc send_MDSCHEMA_CUBES { _cmd } {

  html <s:complexType name="row"><s:choice minOccurs="0" maxOccurs="unbounded">
  pr_ColDef "CATALOG_NAME" "string"
  pr_ColDef "CUBE_NAME" "string"
  html </s:choice></s:complexType></s:schema>

  html {<row></row>}

}



proc send_DISCOVER_PROPERTIES { _cmd } {

  set COLname [list "." "PropertyName" "PropertyDescription" "PropertyType" "PropertyAccessType" "IsRequired" "Value"]

  html <s:complexType name="row"><s:choice minOccurs="0" maxOccurs="unbounded">
  pr_ColDef [lindex $COLname 1] "string"
  pr_ColDef [lindex $COLname 2] "string"
  pr_ColDef [lindex $COLname 3] "string"
  pr_ColDef [lindex $COLname 4] "string"
  pr_ColDef [lindex $COLname 5] "boolean"
  pr_ColDef [lindex $COLname 6] "string"
  html </s:choice></s:complexType></s:schema>

  html <row>
  pr_ColData [lindex $COLname 1] "DataSourceInfo"
  pr_ColData [lindex $COLname 2] ""
  pr_ColData [lindex $COLname 3] "string"
  pr_ColData [lindex $COLname 4] "Read/Write"
  pr_ColData [lindex $COLname 5] "false"
  pr_ColData [lindex $COLname 6] ""
  html </row>
  html <row>
  pr_ColData [lindex $COLname 1] "Catalog"
  pr_ColData [lindex $COLname 2] ""
  pr_ColData [lindex $COLname 3] "string"
  pr_ColData [lindex $COLname 4] "Read/Write"
  pr_ColData [lindex $COLname 5] "false"
  pr_ColData [lindex $COLname 6] ""
  html </row>
  html <row>
  pr_ColData [lindex $COLname 1] "Format"
  pr_ColData [lindex $COLname 2] ""
  pr_ColData [lindex $COLname 3] "string"
  pr_ColData [lindex $COLname 4] "Read/Write"
  pr_ColData [lindex $COLname 5] "false"
  pr_ColData [lindex $COLname 6] "Tabular"
  html </row>

}



proc pr_RequestSchema2 { COLname cmd restr restr_type} {
  html <row>
  pr_ColData [lindex $COLname 1] $cmd
  set _nameAttr [lindex $COLname 2]
  html <$_nameAttr>
  set c [llength $restr]
  if { $c > 0 } {
    html <RestrictionList>
      for {set i 0} {$i < $c} {incr i} {
        set _name [lindex $restr $i]
        set _type [lindex $restr_type $i]
        if { $_type =="1" } {
          html <$_name type="int" />
        } else {
          html <$_name type="string" />
        }
      }
    html </RestrictionList>
  }
  html </$_nameAttr>
  pr_ColData [lindex $COLname 3] ""
  html </row>
}

proc pr_RequestSchema { COLname cmd restr } {
  html <row>
  pr_ColData [lindex $COLname 1] $cmd
  set _nameAttr [lindex $COLname 2]
  html <$_nameAttr>
  set c [llength $restr]
  if { $c > 0 } {
    html <RestrictionList>
      for {set i 0} {$i < $c} {incr i} {
        set _name [lindex $restr $i]
        html <$_name type="string" />
      }
    html </RestrictionList>
  }
  html </$_nameAttr>
  pr_ColData [lindex $COLname 3] ""
  html </row>
}


proc send_DISCOVER_SCHEMA_ROWSETS { _cmd } {

  set COLname [list "." "SchemaName" "Restrictions" "Description"]

  html <s:complexType name="row"><s:choice minOccurs="0" maxOccurs="unbounded">
  pr_ColDef [lindex $COLname 1] "string"
  pr_ColDef [lindex $COLname 2] "any"
  pr_ColDef [lindex $COLname 3] "string"
  html </s:choice></s:complexType></s:schema>

#### FIXME UPDATEME STEP1 register new CALL here

  pr_RequestSchema $COLname "DISCOVER_DATASOURCES" [list "DataSourceName"]
  pr_RequestSchema $COLname "DISCOVER_ENUMERATORS" [list ]
  pr_RequestSchema $COLname "DISCOVER_PROPERTIES" [list ]
  pr_RequestSchema $COLname "DISCOVER_KEYWORDS" [list ]
  pr_RequestSchema $COLname "DISCOVER_LITERALS" [list ]
  pr_RequestSchema $COLname "DISCOVER_PROPERTIES" [list ]
  pr_RequestSchema $COLname "DISCOVER_SCHEMA_ROWSETS" [list ]

  pr_RequestSchema $COLname "DBSCHEMA_CATALOGS" [list ]
  pr_RequestSchema $COLname "MDSCHEMA_CUBES" [list ]

  pr_RequestSchema $COLname "DBSCHEMA_CATALOGS" [list ]
  pr_RequestSchema $COLname "DBSCHEMA_TABLES" [list "TABLE_CATALOG" "TABLE_SCHEMA" "TABLE_NAME" "TABLE_TYPE"]
  pr_RequestSchema $COLname "DBSCHEMA_COLUMNS" [list "TABLE_CATALOG" "TABLE_SCHEMA" "TABLE_SCHEMA" "TABLE_NAME" "COLUMN_NAME"]
  pr_RequestSchema $COLname "DBSCHEMA_PRIMARY_KEYS" [list "TABLE_CATALOG" "TABLE_SCHEMA" "TABLE_NAME"]
  pr_RequestSchema $COLname "DBSCHEMA_FOREIGN_KEYS" [list "PK_TABLE_CATALOG" "PK_TABLE_SCHEMA" "PK_TABLE_NAME" "FK_TABLE_CATALOG" "FK_TABLE_SCHEMA" "FK_TABLE_NAME"]
  pr_RequestSchema2 $COLname "DBSCHEMA_PROVIDER_TYPES" [list "DATA_TYPE"] [list 1]

}



proc send_DISCOVER_ENUMERATORS { _cmd } {

  set COLname [list "." "EnumName" "EnumDescription" "EnumType" "ElementName" "ElementDescription" "ElementValue"]

  html <s:complexType name="row"><s:choice minOccurs="0" maxOccurs="unbounded">
  pr_ColDef [lindex $COLname 1] "string"
  pr_ColDef [lindex $COLname 2] "string"
  pr_ColDef [lindex $COLname 3] "string"
  pr_ColDef [lindex $COLname 4] "string"
  pr_ColDef [lindex $COLname 5] "string"
  pr_ColDef [lindex $COLname 6] "string"
  html </s:choice></s:complexType></s:schema>

  html <row>
  html </row>

}


proc send_DISCOVER_KEYWORDS { _cmd } {

  html <s:complexType name="row"><s:choice minOccurs="0" maxOccurs="unbounded">
  pr_ColDef "Keyword" "string"
  html </s:choice></s:complexType></s:schema>

  html <row>
  html </row>

}


proc pr_Literals { COLname lname lvalue lmax} {
  html <row>
  pr_ColData [lindex $COLname 1] $lname
  if { [string length $lvalue]>0} {
    pr_ColData [lindex $COLname 2] $lvalue
  } else {
    pr_ColData2 [lindex $COLname 2] [list 0 ""]
  }
  pr_ColData2 [lindex $COLname 3] [list 0 ""]
  pr_ColData2 [lindex $COLname 4] [list 0 ""]
  pr_ColData [lindex $COLname 5] $lmax
  html </row>
}

proc send_DISCOVER_LITERALS { _cmd props restr } {
  global _catalogHack

  if {[catch {

        if {[catch { set max_cat [db getinfo  max_catalog_name_len]}]} {
          set max_cat 32
        }
        if {[catch { set max_col [db getinfo  max_column_name_len]}]} {
          set max_col 32
        }
        if {[catch { set max_schema [db getinfo  max_schema_name_len]}]} {
          set max_schema 32
        }
        if {[catch { set max_table [db getinfo  max_table_name_len]}]} {
          set max_table 32
        }
        if {[catch { set cat_sep [db getinfo  catalog_name_separator]}]} {
          set cat_sep "."
        }
        if {[catch { set quote_char [db getinfo  identifier_quote_char]}]} {
           set quote_char "\""
        }
  } err ]} {
      send_Error $err
      return
  }

  set COLname [list "." "LiteralName" "LiteralValue" "LiteralInvalidChars" "LiteralInvalidStartingChars" "LiteralMaxLength"]

  html <s:complexType name="row"><s:choice minOccurs="0" maxOccurs="unbounded">
  pr_ColDef [lindex $COLname 1] "string"
  pr_ColDef [lindex $COLname 2] "string"
  pr_ColDef [lindex $COLname 3] "string"
  pr_ColDef [lindex $COLname 4] "string"
  pr_ColDef [lindex $COLname 5] "int"
  html </s:choice></s:complexType></s:schema>

  pr_Literals $COLname "Catalog_Name" "" $max_cat
  pr_Literals $COLname "Column_Name" "" $max_col
  pr_Literals $COLname "Schema_Name" "" $max_schema
  pr_Literals $COLname "Table_Name" "" $max_table
  pr_Literals $COLname "Catalog_Separator" $cat_sep 1
  pr_Literals $COLname "Schema_Separator" $cat_sep 1
  pr_Literals $COLname "Quote_Prefix" $quote_char 1
  pr_Literals $COLname "Quote_Suffix" $quote_char 1

}


# #################  Execute calls ############################3

proc exec_SCHEMA_CATALOGS { _cmd st cat restr } {
  global _catalogHack

  if { $_catalogHack==1 } {
    send_Empty $_cmd 
    return
  }

    st catalog table_qualifiers

    if {0 != [st numcols]} {

      html <s:complexType name="row"><s:choice minOccurs="0" maxOccurs="unbounded">
      pr_ColDef "CATALOG_NAME" "string"
      pr_ColDef "DESCRIPTION" "string"
      html </s:choice></s:complexType></s:schema>

      set c [st numcols]

      if { [st fetchr]==1 } {
	  while { "1"=="1" } {
	    html <row>

	    pr_ColData2 "CATALOG_NAME"     [st getdatacol2 1] 
	    pr_ColData2 "DESCRIPTION"      [st getdatacol2 5] 

	    html </row>
            if {![st fetchr]} { break }
	  }

      } 
#      else {
#	  html <row></row>
#	  break
#      }

    } else {
      send_Empty $_cmd 
    }
}


proc exec_SCHEMA_TABLES { _cmd st cat restr } {
  global _catalogHack undef

  if { !$_catalogHack } {
    set _cat [getOpt $restr "TABLE_CATALOG" $cat]
  } else {
    set _cat ""
  }

  set _schema [getOpt $restr "TABLE_SCHEMA" ""]
  set _name [getOpt $restr "TABLE_NAME" ""]
  set _type [getOpt $restr "TABLE_TYPE" $undef]

  if {$_type!=$undef} {
      st catalog tables $_cat $_schema $_name $_type
  } else {
      st catalog tables $_cat $_schema $_name
  }

    set c [st numcols]
    if {0 != $c} {
      load_ColsDef st COLname COLtype

      set COLname [lreplace $COLname 1 5 "TABLE_CATALOG" "TABLE_SCHEMA" "TABLE_NAME" "TABLE_TYPE" "REMARKS"]
     
      pr_ColsDef $COLname $COLtype

      if { [st fetchr]==1 } {
	  while { "1"=="1" } {
	    html <row>
	    for {set i 1} {$i <= $c} {incr i} {
	      if {$i == 1} {
	         if { !$_catalogHack } {
	           pr_ColData2 [lindex $COLname 1] [st getdatacol2 1] 
	         } else {
	           pr_ColData2 [lindex $COLname 1] [list 0 {}] 
	         }
	      } else {
	         pr_ColData2 [lindex $COLname $i] [st getdatacol2 $i] 
	      }
	    }
	    html </row>
	    
            if {![st fetchr]} { break }
	  }

      } 
#      else {
#	  html <row></row>
#      }

    } else {
      send_Empty $_cmd 
    }
}



proc exec_SCHEMA_COLUMNS { _cmd st cat restr } {
  global _catalogHack undef

  if { !$_catalogHack } {
    set _cat [getOpt $restr "TABLE_CATALOG" $cat]
  } else {
    set _cat ""
  }

  set _schema [getOpt $restr "TABLE_SCHEMA" ""]
  set _table [getOpt $restr "TABLE_NAME" ""]
  set _column [getOpt $restr "COLUMN_NAME" ""]

    st catalog columns $_cat $_schema $_table $_column

    set c [st numcols]
    if {0 != $c} {
      load_ColsDef st COLname COLtype

      set COLname [lreplace $COLname 1 4 "TABLE_CATALOG" "TABLE_SCHEMA" "TABLE_NAME" "COLUMN_NAME"]
     
      pr_ColsDef $COLname $COLtype

      if { [st fetchr]==1 } {
	  while { "1"=="1" } {
	    html <row>
	    for {set i 1} {$i <= $c} {incr i} {
	      if {$i == 1} {
	         if { !$_catalogHack } {
	           pr_ColData2 [lindex $COLname 1] [st getdatacol2 1] 
	         } else {
	           pr_ColData2 [lindex $COLname 1] [list 0 {}] 
	         }
	      } else {
	         pr_ColData2 [lindex $COLname $i] [st getdatacol2 $i] 
	      }
	    }
	    html </row>

            if {![st fetchr]} { break }
	  }

      } 
#      else {
#	  html <row></row>
#     }

    } else {
      send_Empty $_cmd 
    }
}


proc exec_SCHEMA_PRIMARY_KEYS { _cmd st cat restr } {
  global _catalogHack undef

  if { !$_catalogHack } {
    set _pcat [getOpt $restr "TABLE_CATALOG" $cat]
  } else {
    set _pcat ""
  }

  set _pschema [getOpt $restr "TABLE_SCHEMA" ""]
  set _ptable  [getOpt $restr "TABLE_NAME" ""]

  if { $_ptable!={} } {
    
      st catalog primary_keys $_pcat $_pschema $_ptable

      set c [st numcols]
      load_ColsDef st COLname COLtype

      set COLname [lreplace $COLname 1 3 "TABLE_CATALOG" "TABLE_SCHEMA" "TABLE_NAME"]
     
      pr_ColsDef $COLname $COLtype

      if { [st fetchr]==1 } {
	  while { "1"=="1" } {
	    html <row>
	    for {set i 1} {$i <= $c} {incr i} {
	      if {$i == 1 || $i == 4} {
	         if { !$_catalogHack } {
	           pr_ColData2 [lindex $COLname $i] [st getdatacol2 $i] 
	         } else {
	           pr_ColData2 [lindex $COLname $i] [list 0 {}] 
	         }
	      } else {
	         pr_ColData2 [lindex $COLname $i] [st getdatacol2 $i] 
	      }
	    }
	    html </row>
            if {![st fetchr]} { break }
	  }

      } 
#      else {
#	  html <row></row>
#      }

  } else {

    st catalog tables $_pcat $_pschema "" "TABLE"

    set ltables [st fetchresultset]
    st cursor_close

    set b_loadDef "1"

    if { [llength $ltables] < 1} {
      send_Empty $_cmd 
    }

    foreach entry $ltables {
      set tcat [lindex $entry 0]
      set tsch [lindex $entry 1]
      set ttbl [lindex $entry 2]

      st catalog primary_keys $tcat $tsch $ttbl

      set c [st numcols]
      if {$b_loadDef ==1} {
        set b_loadDef 0

        load_ColsDef st COLname COLtype

        set COLname [lreplace $COLname 1 3 "TABLE_CATALOG" "TABLE_SCHEMA" "TABLE_NAME"]
     
        pr_ColsDef $COLname $COLtype
      }

      if { [st fetchr]==1 } {
	  while { "1"=="1" } {
	    html <row>
	    for {set i 1} {$i <= $c} {incr i} {
	      if {$i == 1} {
	         if { !$_catalogHack } {
	           pr_ColData2 [lindex $COLname $i] [st getdatacol2 $i] 
	         } else {
	           pr_ColData2 [lindex $COLname $i] [list 0 {}] 
	         }
	      } else {
	         pr_ColData2 [lindex $COLname $i] [st getdatacol2 $i] 
	      }
	    }
	    html </row>
            if {![st fetchr]} { break }
	  }

      } 
#      else {
#          if {$b_loadDef == 1} {
# 	    html <row></row>
#          }
#      }
      st cursor_close
    }

  }

}


proc exec_SCHEMA_FOREIGN_KEYS { _cmd st cat restr } {
  global _catalogHack undef

  if { !$_catalogHack } {
    set _pcat [getOpt $restr "PK_TABLE_CATALOG" $cat]
  } else {
    set _pcat ""
  }

  set _pschema [getOpt $restr "PK_TABLE_SCHEMA" ""]
  set _ptable  [getOpt $restr "PK_TABLE_NAME" ""]
  set _fcat    [getOpt $restr "FK_TABLE_CATALOG" ""]
  set _fschema [getOpt $restr "FK_TABLE_SCHEMA" ""]
  set _ftable  [getOpt $restr "FK_TABLE_NAME" ""]

  if { $_ptable!={} } {
    
      st catalog foreign_keys $_pcat $_pschema $_ptable $_fcat $_fschema $_ftable

      set c [st numcols]
      load_ColsDef st COLname COLtype

      set COLname [lreplace $COLname 1 8 "PK_TABLE_CATALOG" "PK_TABLE_SCHEMA" "PK_TABLE_NAME" "PK_COLUMN_NAME" "FK_TABLE_CATALOG" "FK_TABLE_SCHEMA" "FK_TABLE_NAME" "FK_COLUMN_NAME"]
     
      pr_ColsDef $COLname $COLtype

      if { [st fetchr]==1 } {
	  while { "1"=="1" } {
	    html <row>
	    for {set i 1} {$i <= $c} {incr i} {
	      if {$i == 1 || $i == 5} {
	         if { !$_catalogHack } {
	           pr_ColData2 [lindex $COLname $i] [st getdatacol2 $i] 
	         } else {
	           pr_ColData2 [lindex $COLname $i] [list 0 {}] 
	         }
	      } else {
	         pr_ColData2 [lindex $COLname $i] [st getdatacol2 $i] 
	      }
	    }
	    html </row>
            if {![st fetchr]} { break }
	  }

      } 
#      else {
#	  html <row></row>
#      }

  } else {

    st catalog tables $_pcat $_pschema "" "TABLE"

    set ltables [st fetchresultset]
    st cursor_close

    set b_loadDef "1"

    if { [llength $ltables] < 1} {
      send_Empty $_cmd 
    }

    foreach entry $ltables {
      set tcat [lindex $entry 0]
      set tsch [lindex $entry 1]
      set ttbl [lindex $entry 2]

      st catalog foreign_keys $tcat $tsch $ttbl $_fcat $_fschema $_ftable

      set c [st numcols]
      if {$b_loadDef ==1} {
        set b_loadDef 0

        load_ColsDef st COLname COLtype

        set COLname [lreplace $COLname 1 8 "PK_TABLE_CATALOG" "PK_TABLE_SCHEMA" "PK_TABLE_NAME" "PK_COLUMN_NAME" "FK_TABLE_CATALOG" "FK_TABLE_SCHEMA" "FK_TABLE_NAME" "FK_COLUMN_NAME"]
     
        pr_ColsDef $COLname $COLtype
      }

      if { [st fetchr]==1 } {
	  while { "1"=="1" } {
	    html <row>
	    for {set i 1} {$i <= $c} {incr i} {
	      if {$i == 1 || $i == 5} {
	         if { !$_catalogHack } {
	           pr_ColData2 [lindex $COLname $i] [st getdatacol2 $i] 
	         } else {
	           pr_ColData2 [lindex $COLname $i] [list 0 {}] 
	         }
	      } else {
	         pr_ColData2 [lindex $COLname $i] [st getdatacol2 $i] 
	      }
	    }
	    html </row>
            if {![st fetchr]} { break }
	  }

      } 
#      else {
#          if {$b_loadDef == 1} {
# 	    html <row></row>
#          }
#      }
      st cursor_close
    }

  }

}



proc exec_SCHEMA_PROVIDER_TYPES { _cmd st cat restr } {

  set _type [getOpt $restr "DATA_TYPE" "0"]

    st catalog datatypes $_type

    set c [st numcols]
    if {0 != $c} {
      load_ColsDef st COLname COLtype
      pr_ColsDef $COLname $COLtype

      if { [st fetchr]==1 } {
	  while { "1"=="1" } {
	    html <row>
	    for {set i 1} {$i <= $c} {incr i} {
	      pr_ColData2 [lindex $COLname $i] [st getdatacol2 $i] 
	    }
	    html </row>

            if {![st fetchr]} { break }
	  }

      }
#       else {
#	  html <row></row>
#      }

    } else {
      send_Empty $_cmd 
    }
}







proc exec_req { cmd query props restr isSQL } {

    set CAT [getOpt $props "Catalog" ""]
    set _executed 0

#### FIXME UPDATEME STEP2 register new CALL here
    set _req [string trimleft $query]

    if { $isSQL==0 } {

      if {[catch {
        switch -- $_req {
           "DISCOVER_DATASOURCES" {
               send_Datasources $cmd
               set _executed 1
               }
           "MDSCHEMA_CUBES" {
               send_MDSCHEMA_CUBES $cmd
               set _executed 1
               }
           "DISCOVER_PROPERTIES" {
               send_DISCOVER_PROPERTIES $cmd
               set _executed 1
               }
           "DISCOVER_SCHEMA_ROWSETS" {
               send_DISCOVER_SCHEMA_ROWSETS $cmd
               set _executed 1
               }
           "DISCOVER_ENUMERATORS" {
               send_DISCOVER_ENUMERATORS $cmd
               set _executed 1
               }
           "DISCOVER_KEYWORDS" {
               send_DISCOVER_KEYWORDS $cmd
               set _executed 1
               }
        }
      } err ]} {
        send_Error $err
        return
      }
      if { $_executed==1} {
         return
      }

          
      if {[catch {
          connect_db db $props
      } err ]} {
        send_Error $err
        catch {db close}
        return
      }

      if {[catch {
  	db statement st
      } err ]} {
        send_Error $err
        catch {st close}
        catch {db close}
        return
      }

      if {[catch {
        switch -- $_req {
          "DISCOVER_LITERALS" {
               send_DISCOVER_LITERALS $cmd $props $restr
               }
          "DBSCHEMA_CATALOGS" { 
      		exec_SCHEMA_CATALOGS $cmd st $CAT $restr
		}
          "DBSCHEMA_TABLES" { 
      		exec_SCHEMA_TABLES $cmd st $CAT $restr
		}
          "DBSCHEMA_VIEWS" { 
                send_Empty $cmd 
		}
          "DBSCHEMA_COLUMNS" { 
      		exec_SCHEMA_COLUMNS $cmd st $CAT $restr
		}
          "DBSCHEMA_PRIMARY_KEYS" { 
      		exec_SCHEMA_PRIMARY_KEYS $cmd st $CAT $restr
		}
          "DBSCHEMA_FOREIGN_KEYS" { 
      		exec_SCHEMA_FOREIGN_KEYS $cmd st $CAT $restr
		}
          "DBSCHEMA_PROVIDER_TYPES" { 
      		  exec_SCHEMA_PROVIDER_TYPES $cmd st $CAT $restr
		}
          default { 
                send_Empty $cmd 
		}
        }
      } err ]} {
        send_Error $err
      }

      catch {st close}
      catch {db close}
      return
    
    }
    

    if {[catch {
        connect_db db $props
    } err ]} {
      send_Error $err
      catch {db close}
      return
    }

    if {[catch {

  	db statement st
  	if {[string length $CAT] > 0} {
  	  catch { db option current_catalog $CAT }
  	}
	st prepare $query
	st execute

        set c [st numcols]
        if {0 != $c} {

          while { "1"== "1"} {

            load_ColsDef st COLname COLtype
            pr_ColsDef $COLname $COLtype

	    if { [st fetchr]==1 } {
	      while { "1"=="1" } {
	        html <row>
	        for {set i 1} {$i <= $c} {incr i} {
	          pr_ColData2 [lindex $COLname $i] [st getdatacol2 $i]
	        }
	        html </row>
                if {![st fetchr]} { break }
	      }

	      if { ![st moreresults]} { break }
	    } else {
#	      html <row></row>
	      break
	    }
          }
        } else {
          send_Empty $cmd 
        }

    
    } err ]} {
      send_Error $err
    }

    catch {st close}
    catch {db close}
}


proc handlepost {} {
    global undef _close_soap_answer
    var myReq

    set cReq $undef
    set cParam ""
    set cProps "Properties"
    set cRestr "Restrictions"

    parse_cmd cReq cParam cProps cRestr

    
    # needs update permissions
 #    if {![check_auth 2]} return
 #   if {![TXvalid wod]} return

#### FIXME UPDATEME STEP3 register new CALL here, if it is nedded

    if {$cReq == "Discover"} {
        pr_SoapHeader
        pr_SoapEnvelopeBegin
        pr_SoapBodyBegin "Discover"

        exec_req $cReq $cParam $cProps $cRestr 0

        if {$_close_soap_answer == 1} {
          pr_SoapBodyEnd "Discover"
          pr_SoapEnvelopeEnd
        }
    } elseif {$cReq == "Execute"} {

        pr_SoapHeader
        pr_SoapEnvelopeBegin
        pr_SoapBodyBegin "Execute"

        if { [string length $cParam] > 0 && [string index $cParam 0]=="*"} {
          set _CMD [string range $cParam 1 end]
  
          exec_req $cReq $_CMD $cProps $cRestr 0

        } else {
          exec_req $cReq $cParam $cProps $cRestr 1
        }

        if {$_close_soap_answer == 1} {
          pr_SoapBodyEnd "Execute"
          pr_SoapEnvelopeEnd
        }
    }
}



proc show_wsdl { } {
  set cmd [string tolower [req args]]

  if {$cmd == "wsdl"} {
    if {[catch {
       set fid [open "Service.xml" "r"]
       set data [read $fid]
       close $fid
       html $data
    } err ]} {
       send_Error $err
    }
  } else {
    send_Empty "Empty"
  }

}


#
#  Main
#
#if {![check_auth 1]} return

nocache

if {[req method] == "POST"} {
    handlepost
} else {
    show_wsdl
}
